  function get_errtxt(e_no: integer):  string;
  begin
  case e_no of
    C_OPEN:           get_errtxt:= 'Can''t open source file';
    I_MODE:           get_errtxt:= 'Illegal addr. mode';
    G_SYNT:           get_errtxt:= 'General syntax error';
    C_OPEN_LST:       get_errtxt:= 'Can''t open listing file';
    OUT_RADIX:        get_errtxt:= 'Digit out of radix';
    TOO_LARGE_NUM:    get_errtxt:= 'Integer overflow';
    ILL_LAB:          get_errtxt:= 'Illegal name';
    REDEF:            get_errtxt:= 'Redefinition of symbol <' + redef_ident +'>';
    EXP_EXPR:         get_errtxt:= 'Expected expression';
    ILL_CHAR:         get_errtxt:= 'Illegal character in text';
    C_BRACKED:        get_errtxt:= 'Exp. close bracked';
    NO_MEM:           get_errtxt:= 'There is no memory available';
    OUT_OVERFL:       get_errtxt:= 'Output buf. overflow';
    BRANCH_OUT:       get_errtxt:= 'Branch out of range';
    OBJ_ERR:          get_errtxt:= 'Error writing obj. file';
    NOT_FOUND:        get_errtxt:= 'Undefined identifier <' + undef_ident +'>';
    C_OPEN_SRC:       get_errtxt:= 'Can''t open source file';
    EXP_CONST:        get_errtxt:= 'Expected constant expression';
    ILL_SIZE:         get_errtxt:= 'Illegal size of operand';
    EXP_CLOSE_QUOTE:  get_errtxt:= 'Expected close quote';
    BAD_RADIX:        get_errtxt:= 'Bad radix specified';
    NESTING_ERR:      get_errtxt:= 'Proc nesting error';
    PASS_ERR:         get_errtxt:= 'Error betwen passes';
    NO_SEC:           get_errtxt:= 'ENDS without SECTION';
    NO_ENDS:          get_errtxt:= 'Missing ENDS';
    BAD_ORG:          get_errtxt:= 'ORG inside SECTION';
    INV_ORG:          get_errtxt:= 'ORG not allowed in relative mode';
    INV_EXT:          get_errtxt:= 'EXTERN not allowed in not relative mode';
    INV_PBL:          get_errtxt:= 'PUBLIC not allowed in not relative mode';
    ILL_CSUM:         get_errtxt:= 'Improper use of CHECKSUM directive';
(**)IncNested:        get_errtxt:= 'Include files too nested';
(**)c_open_inc:       get_errtxt:= 'Can''t open include file';
(**)FnameExpect:      get_errtxt:= 'Include file name expected';
(**)UserBreak:        get_errtxt:= 'User error = ' + UsrBrk_data;
(**)EndExpect:        get_errtxt:= 'Unexpected directive END';
    ILL_EXPR:         get_errtxt:= 'Illegal expression';
    ENDIFexpect:      get_errtxt:= '.ENDIF expected';
    DEFnameExpect:    get_errtxt:= 'definition symbol expected';
    TooManyDEF:       get_errtxt:= 'too many symbols for conditional compilation';
    CONDmsplc:        get_errtxt:= 'misplaced conditional directive';
    end; { case }
  end;

  procedure draw_err(e_no, e_lvl: integer);
    var s: string[20];
  begin
  s:= ',  Error: ';
  if e_lvl > 1 then s:= ',  Warning: ';
  if NotCon
    then begin write(#13);
         if not errfile then rewrite(StdErr);
         errfile:= true;
         writeln(StdErr, inc_fname[IncLevel], '(', IncLines[IncLevel], ')',
                          s, get_errtxt(e_no), '.');
         writeln(StdOut, inc_fname[IncLevel], '(', IncLines[IncLevel], ')',
                          s, get_errtxt(e_no), '.');
         write(#13, inc_fname[IncLevel], '(', IncLines[IncLevel], ')'#13);
         end;
  if has_lst (* and gen_list *)
    then writeln(lst, inc_fname[IncLevel], '(', IncLines[IncLevel], ')',
                      s, get_errtxt(e_no), '.');
  end { draw_err };

  procedure error(e_no, level: integer); { 0 - fatal, 1 - Severe, 2 - Warning }
  begin
  if level = 0
    then begin writeln;           { Fatal errors reported directly to console }
         if Inc_Fname[0] <> ''                 (*     ? *)
           then write(inc_fname[IncLevel], '(', IncLines[IncLevel], ')');
         write('Fatal Error: ', get_errtxt(e_no), '.');
         if errfile then Close(StdErr) else erase(StdErr);
         Close(StdOut);
         halt(2);
         end;
  if (l_errors < 10) and ((errors[l_errors-1] <>  e_no) or (l_errors = 0))
    then begin errors[l_errors]:= e_no; err_levels[l_errors]:= level;
         if level = 1
           then inc(n_errors)
           else inc(n_warns);
         inc(l_errors);
         end;
  end; { error }

  procedure display_err;
    var i: integer;
  begin
  for i:= 0 to l_errors-1 do draw_err(errors[i], err_levels[i]);
  end; { display_err }

